using System;
using System.IO;
using System.Globalization;
using System.Collections;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace RSSStoreViewer
{
	/// <summary>
	/// Summary description for Feed.
	/// </summary>
    public class Feed
    {
        protected String m_path;
        protected int m_index;
        protected int m_itemCount; // known by feed store itself
        protected ArrayList m_feedItems; // actual items
        protected bool m_bDirty;

        public Feed(int index) 
        {
            m_index = index;
            m_itemCount = 0;
            m_bDirty = false;

            m_feedItems = new ArrayList();
        }
        
        public String Path 
        {
            get {
                return m_path;
            }
            set {
                m_path = value;
            }
        }

        public int ItemCount 
        {
            get 
            {
                return m_itemCount;
            }
            set 
            {
                m_itemCount = value;
            }
        }

        public ArrayList Items
        {
            get
            {
                return m_feedItems;
            }
        }

        public FeedItem this [int index]
        {
            get 
            {
                if (index < 0 || index >= m_feedItems.Count)
                    return null;
                return (FeedItem) m_feedItems[index];
            }
        }
        

        public bool Dirty
        {
            get 
            {
                return m_bDirty;
            }
            set
            {
                m_bDirty = value;
            }
        }

        public bool Read(byte[] stream)
        {
            bool bXmlParsingStatus = false;

            MemoryStream m = new MemoryStream(stream);

			System.Xml.XmlTextReader reader = null;

			try
			{  
				reader = new System.Xml.XmlTextReader(m);
				reader.WhitespaceHandling = System.Xml.WhitespaceHandling.None;
    
                bool bReadCount = false;

                while ( reader.Read() )
                {
                    switch ( reader.NodeType )
                    {
                        case System.Xml.XmlNodeType.Element:
                        {
                            bReadCount = false;
                            if (reader.Name == "FeedDataCache" && reader.AttributeCount > 0)
                            {
                                reader.MoveToAttribute(0);
                                for (int i = 0; i < reader.AttributeCount; i++)
                                {
                                    reader.MoveToAttribute(i);
                                    if (reader.Name == "Path")
                                    {
                                        m_path = reader.Value;
                                        bXmlParsingStatus = true;
                                    }
                                }
                                reader.MoveToElement();
                            }
                            else if (reader.Name == "Count")
                            {
                                bReadCount = true;
                            }
                        }break;
                        case System.Xml.XmlNodeType.Text:
                        {
                            if (bReadCount)
                            {
                                m_itemCount = Int32.Parse(reader.Value.Replace("0x",""), 
                                                          NumberStyles.AllowHexSpecifier);
                                bReadCount = false;
                            }
                        }break;

                    }       
                } // end while      
            }
            catch( System.Xml.XmlException)
            {
                //MessageBox.Show("exception : " + e.ToString());
                bXmlParsingStatus = false;
            }
            finally
            {
                if (reader != null) 
                    reader.Close();
            }

            m.Close();

            return bXmlParsingStatus;
        }


        public bool Open(String rootpath)
        {
            if (m_feedItems.Count > 0 && !m_bDirty)
                return true;

            m_feedItems.Clear();
            m_bDirty = false;

            String feedpath = rootpath;
            feedpath += this.m_path.Replace(".","~d").Replace(":","~c").Replace(@"\",@"~\"); //TODO : find other escape sequences
            feedpath += "~.feed-ms";

            // open the storage
            // enum all streams
            //  retain those which have a numeric name, i.e. 0, 1, ...
            //  read streams into Feed item objects
            //  close streams
            // close storage
    
            int nbFailedStreams = 0;
            int nbStreams = 0;

            NativeCalls.IStorage strge = null;
            Int32 hr = NativeCalls.StgOpenStorage(feedpath, 
                (NativeCalls.IStorage)null, 
                NativeCalls.STGM_READ | NativeCalls.STGM_SHARE_EXCLUSIVE, 
                IntPtr.Zero,
                0, 
                out strge);

            if (strge == null)
            {
                MessageBox.Show(String.Format("Sync problem. Feed '{0}' does not appear to exist.", this.m_path));
                return false;
            }

            NativeCalls.IEnumSTATSTG penum = null;

            strge.EnumElements((uint)0,
                IntPtr.Zero,
                (uint)0,
                out penum);
            if (penum == null)
            {
                Marshal.ReleaseComObject(strge);
                return false;
            }

            penum.Reset();


            uint nb = 0;

            do
            {
                nb = 0;
                STATSTG[] statstg = new STATSTG[1];
                penum.Next(1, statstg, out nb);
                if (nb > 0 &&
                    statstg[0].type == NativeCalls.STGTY_STREAM)
                {
                    UCOMIStream ppstm = null;

                    //make sure the name is a number
                    bool bCandidateStream = true;
                    try
                    {
                        int d = Int32.Parse(statstg[0].pwcsName);
                    }
                    catch (Exception)
                    {
                        bCandidateStream = false;
                    }

                    if (!bCandidateStream)
                        continue;

                    strge.OpenStream(statstg[0].pwcsName,
                        IntPtr.Zero/* reserved1*/,
                        NativeCalls.STGM_READ | NativeCalls.STGM_SHARE_EXCLUSIVE,
                        (uint)0/* reserved2*/,
                        out ppstm);
                    if (ppstm != null)
                    {
                        STATSTG statstgstream;
                        ppstm.Stat(out statstgstream, 1);
                        byte[] buffer = new byte[statstgstream.cbSize];
                        GCHandle handle = GCHandle.Alloc(buffer, GCHandleType.Pinned);
                        ppstm.Read(buffer, (int) statstgstream.cbSize, IntPtr.Zero);

                        FeedItem item = new FeedItem(statstg[0].pwcsName, m_feedItems.Count);
                        if (!item.Read(buffer))
                            nbFailedStreams++;
                        else
                            m_feedItems.Add(item);

                        handle.Free();
                        Marshal.ReleaseComObject(ppstm);
                    }
                    else
                    {
                        nbFailedStreams++;
                    }

                    nbStreams++;
                }
            } while (nb > 0);

            Marshal.ReleaseComObject(penum);

            Marshal.ReleaseComObject(strge);

            if (nbFailedStreams > 0 && nbStreams > 0)
            {
                MessageBox.Show (String.Format("{0} out of {1} feed items could not be read for internal file format error reasons.", 
                    nbFailedStreams,
                    nbStreams));
                return false;
            }

            return true;
        }

    }

}
